# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import tempfile
import urlparse

from boto.s3.connection import S3Connection


def make_s3_connection(aws_access_key_id=None, aws_secret_access_key=None):
    """Mockable point for creating S3Connections."""
    return S3Connection(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )


class s3file(object):
    def __new__(
        cls,
        filename,
        mode='r',
        aws_access_key_id=None,
        aws_secret_access_key=None,
    ):
        """Opens a local copy of an S3 URL."""
        parse = urlparse.urlparse(filename)
        if 'w' in mode:
            raise ValueError("can't write to S3")
        if parse.scheme != 's3':
            raise ValueError("s3file can't open non-S3 URLs")

        conn = make_s3_connection(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )
        bucket = conn.get_bucket(parse.netloc)
        key = bucket.get_key(parse.path)

        local_file = tempfile.TemporaryFile()
        key.get_contents_to_file(local_file)
        local_file.seek(0)
        return local_file



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import bz2
import gzip
import urllib2
import urlparse

from canopener.s3file import s3file


class WithMixin(object):
    """Gives a file-like object `with` powers."""

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()


class bz2file(bz2.BZ2File, WithMixin):
    """Wrapper around bz2.BZ2File that imparts `with` powers.

    Only required in Python <= 2.6.
    """
    pass


class gzfile(gzip.GzipFile, WithMixin):
    """Wrapper around gzip.GzipFile that imparts `with` powers.

    Only required in Python <= 2.6.
    """

    def close(self):
        super(gzfile, self).close()
        # Close the underlying fileobj if it exists. gzip.GzipFile
        # doesn't do this by default to enable writing uncompressed
        # sections to a file.
        if self.fileobj is not None:
            self.fileobj.close()


class canopener(object):
    """Convenience opener. Seamlessly decompresses .gz and .bz2 URLs and
    files.
    """

    def __new__(
        cls,
        filename,
        mode='r',
        aws_access_key_id=None,
        aws_secret_access_key=None
    ):
        # Open the base file stream.
        parse = urlparse.urlparse(filename)
        if parse.netloc:
            if 'w' in mode:
                raise ValueError("can't write to URLs")
            if parse.scheme == 's3':
                base_file = s3file(
                    filename,
                    mode,
                    aws_access_key_id=aws_access_key_id,
                    aws_secret_access_key=aws_secret_access_key,
                )
            else:
                base_file = urllib2.urlopen(filename)
        else:
            base_file = open(filename, mode)

        # Now wrap it in any decompression.
        if filename.endswith('.gz'):
            return gzfile(fileobj=base_file, mode=mode)
        elif filename.endswith('.bz2'):
            return bz2file(base_file, mode)
        else:
            return base_file



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from setuptools import setup


setup(
    name='canopener',
    version='0.1.1',
    author='David Selassie',
    author_email='selassid@gmail.com',
    package='canopener',
    url='https://github.com/selassid/canopener',
    license='LICENSE.txt',
    description=(
        'Python convenience function for opening compressed URLs and files.'
    ),
    long_description=open('README.txt').read(),
    setup_requires=['setuptools'],
    install_requires=[
        'boto',
        'pystaticconfiguration',
    ],
)
